#!/usr/freeware/medx/bin/tixwish

# Stuart Clare, FMRIB Physics Group
#
# Copyright (C) 1998-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

# paths
if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

set BINPATH ${FSLDIR}/bin
set HTMLPATH ${FSLDIR}/doc/anova

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc anova:dialog { w } {

    global PXHOME
    global FSLDIR
    global HTMLPATH
    global anova_disable_buttons

    toplevel $w
    wm title $w "ANOVA fMRI Analysis"
    wm iconname $w "ANOVA"
    wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

    tixNoteBook $w.nb -ipadx 5 -ipady 5
    $w.nb add cyclic -label "Cyclic ANOVA"
    $w.nb add noncyc -label "Triggered ANOVA"

    #-------- Cyclic --------
    set cyclicf [$w.nb subwidget cyclic]
    
    frame $w.cyc
    label $w.cyc.label -text "Threshold (% of max):"
    tixControl $w.cyc.thold \
	    -integer true \
	    -variable ANOVAvariables(thold) \
	    -min 0 \
	    -selectmode immediate \
	    -options { entry.width 5 }
    label $w.cyc.label1 -text "Ignore first n volumes:"
    entry $w.cyc.offset -textvariable ANOVAvariables(offset) -width 5
    label $w.cyc.label2 -text "Volumes per complete cycle:"
    entry $w.cyc.volcyc -textvariable ANOVAvariables(volcyc) -width 5

    label $w.cyc.label3 -text "Timecourse:"
    tixOptionMenu $w.cyc.av -variable ANOVAvariables(timecourse)

    $w.cyc.av add command 0 -label "Whole"
    $w.cyc.av add command 1 -label "Average"
    $w.cyc.av add command 2 -label "None"

    grid $w.cyc.label -row 1 -column 1 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.thold -row 1 -column 2 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.label1 -row 2 -column 1 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.offset -row 2 -column 2 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.label2 -row 3 -column 1 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.volcyc -row 3 -column 2 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.label3 -row 4 -column 1 -sticky w -in $w.cyc -padx 3 -pady 3
    grid $w.cyc.av -row 4 -column 2 -sticky w -in $w.cyc -padx 3 -pady 3

    pack $w.cyc -in $cyclicf -side top -padx 5 -pady 5 -anchor nw
    
    #-------- Non-Cyclic --------
    set noncycf [$w.nb subwidget noncyc]
    
    frame $w.non

    label $w.non.label -text "Threshold (% of max):"
    tixControl $w.non.thold \
	    -integer true \
	    -variable ANOVAvariables(thold) \
	    -min 0 \
	    -selectmode immediate \
	    -options { entry.width 5 }    
    label $w.non.label1 -text "Number of events:  "
    entry $w.non.numev -textvariable ANOVAvariables(numev) -width 5
    label $w.non.label2 -text "Volumes per event:"
    entry $w.non.volev -textvariable ANOVAvariables(volev) -width 5
    label $w.non.label3 -text "File of events:"
    tixFileEntry $w.non.file \
        -variable ANOVAvariables(evfile) \
        -dialogtype tixFileSelectDialog \
        -options { entry.width 25 }
    $w.non.file filedialog subwidget fsbox config -directory ~ 

    button $w.non.edit \
	    -text "Edit" \
	    -width 5 \
	    -command "anova:editfile $w"

    label $w.non.label4 -text "Timecourse:"
    tixOptionMenu $w.non.av -variable ANOVAvariables(timecourse)

    $w.non.av add command 0 -label "Whole"
    $w.non.av add command 1 -label "Average"
    $w.non.av add command 2 -label "None"

    grid $w.non.label -row 1 -column 1 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.thold -row 1 -column 2 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.label1 -row 2 -column 1 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.numev -row 2 -column 2 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.label2 -row 3 -column 1 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.volev -row 3 -column 2 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.label3 -row 4 -column 1 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.file -row 4 -column 2 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.edit -row 5 -column 2 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.label4 -row 6 -column 1 -sticky w -in $w.non -padx 3 -pady 3
    grid $w.non.av -row 6 -column 2 -sticky w -in $w.non -padx 3 -pady 3

    pack $w.non -in $noncycf -side top -padx 5 -pady 5 -anchor nw
    
    #-------- Buttons --------
    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1

    button $w.ok \
	-text "OK" -width 5 \
	-command "anova:apply $w destroy" 
    bind $w.ok <Return> {
	[winfo toplevel %W].ok invoke}

    button $w.apply	-command "anova:apply $w keep" \
	-text "Apply" -width 5
    bind $w.apply <Return> {
	[winfo toplevel %W].apply invoke}

    button $w.cancel	-command "anova:destroy $w" \
	-text "Cancel" -width 5
    bind $w.cancel <Return> {
	[winfo toplevel %W].cancel invoke}

	button $w.help	-command "FmribWebHelp file: ${HTMLPATH}/index.html" \
	-text "Help" -width 5
    bind $w.help <Return> {
	[winfo toplevel %W].help invoke}

    pack $w.btns.b -side bottom -fill x -padx 3 -pady 5
    pack $w.ok $w.apply $w.cancel $w.help -in $w.btns.b \
	-side left -expand yes -padx 3 -pady 10 -fill y

	if { $anova_disable_buttons } {
	    pack forget $w.apply
	    pack forget $w.cancel
	}
	
    pack $w.nb $w.btns -expand yes -fill both
}
proc anova:apply { w dialog } {

    global ANOVAvariables

    $w.non.file update

    set page [$w.nb raised]
    set result [anova:process_$page $w]

    update idletasks

    if {$dialog == "destroy"} {
	if {$result == "done"} {
	    anova:destroy $w
	}
    }
}
proc anova:destroy { w } {
    destroy $w
}
proc anova:process_cyclic { w } {

    global ANOVAvariables
    global BINPATH

    if { $ANOVAvariables(thold) =="" } {
	set Err "Threshold not specified"
	puts $Err
	MxPause $Err
	return	
    }
    if { $ANOVAvariables(volcyc) =="" } {
	set Err "Volumes per cycle not specified"
	puts $Err
	MxPause $Err
	return	
    }
    if { $ANOVAvariables(offset) =="" } {
	set Err "Offset not specified"
	puts $Err
	MxPause $Err
	return	
    }

    ScriptUpdate "Running Medx ANOVA.\nPlease Wait."
    puts "Starting ANOVA"

    set PROCID [pid]
    incr ANOVAvariables(NUM) 1
    set OUTNAME $PROCID-$ANOVAvariables(NUM)
    incr ANOVAvariables(NUM) 1
    set FMAP $PROCID-$ANOVAvariables(NUM)
    incr ANOVAvariables(NUM) 1
    set ZMAP $PROCID-$ANOVAvariables(NUM)
    incr ANOVAvariables(NUM) 1
    set TIMECOURSE $PROCID-$ANOVAvariables(NUM)

    # Save current data
    MxGetCurrentGroup Group
    MxSetTimeSeriesProperties $Group 1 red
    MxGetCurrentFolder Folder
    MxSetFolderProperties $Folder {{SaveAll true}}
    FSLSaveAs $Group AVW "/tmp/$OUTNAME.hdr" true

    # Run medx_anova
    if { $ANOVAvariables(timecourse) == 1 } {
	set PROCSTRING "$ANOVAvariables(thold) $ANOVAvariables(offset) $ANOVAvariables(volcyc) -av /tmp/$TIMECOURSE"
    } else {
	set PROCSTRING "$ANOVAvariables(thold) $ANOVAvariables(offset) $ANOVAvariables(volcyc)"
    }
    set op [open "| ${BINPATH}/medx_anova /tmp/$OUTNAME /tmp/$FMAP /tmp/$ZMAP $PROCSTRING"]
    while {[gets $op Err] >=0 } {
	CancelScriptUpdate
	puts $Err
	MxPause $Err
	anova:cleanup $OUTNAME
	anova:cleanup $FMAP
	anova:cleanup $ZMAP
	return
    }

    if { $ANOVAvariables(timecourse) == 0 } {
	# Open result
	MxGetCurrentPage Page
	MxGetPageProperties $Page list
	set PageName [keylget list Name]
	keylset improps Name "ANOVA F-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$FMAP.img I0
	keylset improps Name "ANOVA Z-Map of $PageName ORIG"
	MxOpenImage $Folder $improps /tmp/$ZMAP.img I1
	
	# create group SD (not mean, as SD is float) in which to place Z results
	MxWithinGrpStatistics $Group S I2
	MxExpr $I2 = $I1
	MxDeleteImage $I1
	MxGetImageProperties $I2 sdimageprops
	keylset sdimageprops Name "ANOVA Z-Map of $PageName"
	MxSetImageProperties $I2 sdimageprops
	MxGroupImages {$I0 $I2} I "ANOVA fMRI analysis of $PageName" NEWGROUP
    }

    if { $ANOVAvariables(timecourse) == 1 } {
	# Open result
	MxGetCurrentPage Page
	MxGetPageProperties $Page list
	set PageName [keylget list Name]
	keylset improps Name "ANOVA F-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$FMAP.img I0
	keylset improps Name "ANOVA Z-Map of $PageName ORIG"
	MxOpenImage $Folder $improps /tmp/$ZMAP.img I1
	keylset improps Name "Average timecourse of $PageName"
	MxOpenImage $Folder $improps /tmp/$TIMECOURSE.img I2
	MxSetTimeSeriesProperties $I2 0 black
	MxWithinGrpStatistics $I2 S I3
	MxExpr $I3 = $I1
	MxDeleteImage $I1
	MxGetImageProperties $I3 sdimageprops
	keylset sdimageprops Name "ANOVA Z-Map of $PageName"
	MxSetImageProperties $I3 sdimageprops
	MxGroupImages {$I0 $I3} I "ANOVA fMRI analysis of $PageName" NEWGROUP

    }

    if { $ANOVAvariables(timecourse) == 2 } {
	# Open result
	MxGetCurrentPage Page
	MxGetPageProperties $Page list
	set PageName [keylget list Name]
	keylset improps Name "ANOVA F-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$FMAP.img I0
	keylset improps Name "ANOVA Z-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$ZMAP.img I1
	MxGroupImages {$I0 $I1} I "ANOVA fMRI analysis of $PageName" NEWGROUP
    }

    # Delete temporary file
    anova:cleanup $OUTNAME
    anova:cleanup $FMAP
    anova:cleanup $ZMAP

    CancelScriptUpdate
    
    puts "ANOVA Done"
    return "done"
}
proc anova:process_noncyc { w } {

    global ANOVAvariables
    global BINPATH

    if { $ANOVAvariables(thold) =="" } {
	set Err "Threshold not specified"
	puts $Err
	MxPause $Err
	return	
    }
    if { $ANOVAvariables(volev) =="" } {
	set Err "Volumes per event not specified"
	puts $Err
	MxPause $Err
	return	
    }
    if { $ANOVAvariables(numev) =="" } {
	set Err "Number of events not specified"
	puts $Err
	MxPause $Err
	return	
    }
    if { $ANOVAvariables(evfile) =="" } {
	set Err "File of events not specified"
	puts $Err
	MxPause $Err
	return	
    }

    ScriptUpdate "Running Medx ANOVA.\nPlease Wait."
    puts "Starting ANOVA"

    set PROCID [pid]
    incr ANOVAvariables(NUM) 1
    set OUTNAME $PROCID-$ANOVAvariables(NUM)
    incr ANOVAvariables(NUM) 1
    set FMAP $PROCID-$ANOVAvariables(NUM)
    incr ANOVAvariables(NUM) 1
    set ZMAP $PROCID-$ANOVAvariables(NUM)
    incr ANOVAvariables(NUM) 1
    set TIMECOURSE $PROCID-$ANOVAvariables(NUM)

    # Save current data
    MxGetCurrentGroup Group
    MxSetTimeSeriesProperties $Group 1 red
    MxGetCurrentFolder Folder
    MxSetFolderProperties $Folder {{SaveAll true}}
    FSLSaveAs $Group AVW "/tmp/$OUTNAME.hdr" true

    # Run medx_anova
    if { $ANOVAvariables(timecourse) == 1 } {
	set PROCSTRING "$ANOVAvariables(thold) $ANOVAvariables(numev) $ANOVAvariables(volev) $ANOVAvariables(evfile) -av /tmp/$TIMECOURSE"
    } else {
	set PROCSTRING "$ANOVAvariables(thold) $ANOVAvariables(numev) $ANOVAvariables(volev) $ANOVAvariables(evfile)"
    }
    
    set op [open "| ${BINPATH}/medx_anovanc /tmp/$OUTNAME /tmp/$FMAP /tmp/$ZMAP $PROCSTRING"]
    while {[gets $op Err] >=0 } {
	CancelScriptUpdate
	puts $Err
	MxPause $Err
	anova:cleanup $OUTNAME
	anova:cleanup $FMAP
	return
    }


    if { $ANOVAvariables(timecourse) == 0 } {
	# Open result
	MxGetCurrentPage Page
	MxGetPageProperties $Page list
	set PageName [keylget list Name]
	keylset improps Name "ANOVA F-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$FMAP.img I0
	keylset improps Name "ANOVA Z-Map of $PageName ORIG"
	MxOpenImage $Folder $improps /tmp/$ZMAP.img I1
	
	# create group SD (not mean, as SD is float) in which to place Z results
	MxWithinGrpStatistics $Group S I2
	MxExpr $I2 = $I1
	MxDeleteImage $I1
	MxGetImageProperties $I2 sdimageprops
	keylset sdimageprops Name "ANOVA Z-Map of $PageName"
	MxSetImageProperties $I2 sdimageprops
	MxGroupImages {$I0 $I2} I "ANOVA fMRI analysis of $PageName" NEWGROUP
    }

    if { $ANOVAvariables(timecourse) == 1 } {
	# Open result
	MxGetCurrentPage Page
	MxGetPageProperties $Page list
	set PageName [keylget list Name]
	keylset improps Name "ANOVA F-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$FMAP.img I0
	keylset improps Name "ANOVA Z-Map of $PageName ORIG"
	MxOpenImage $Folder $improps /tmp/$ZMAP.img I1
	keylset improps Name "Average timecourse of $PageName"
	MxOpenImage $Folder $improps /tmp/$TIMECOURSE.img I2
	MxSetTimeSeriesProperties $I2 0 black
	MxWithinGrpStatistics $I2 S I3
	MxExpr $I3 = $I1
	MxDeleteImage $I1
	MxGetImageProperties $I3 sdimageprops
	keylset sdimageprops Name "ANOVA Z-Map of $PageName"
	MxSetImageProperties $I3 sdimageprops
	MxGroupImages {$I0 $I3} I "ANOVA fMRI analysis of $PageName" NEWGROUP

    }

    if { $ANOVAvariables(timecourse) == 2 } {
	# Open result
	MxGetCurrentPage Page
	MxGetPageProperties $Page list
	set PageName [keylget list Name]
	keylset improps Name "ANOVA F-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$FMAP.img I0
	keylset improps Name "ANOVA Z-Map of $PageName"
	MxOpenImage $Folder $improps /tmp/$ZMAP.img I1
	MxGroupImages {$I0 $I1} I "ANOVA fMRI analysis of $PageName" NEWGROUP
    }

    # Delete temporary file
    anova:cleanup $OUTNAME
    anova:cleanup $FMAP
    anova:cleanup $ZMAP

    CancelScriptUpdate
    
    puts "ANOVA Done"
    return "done"
}
proc anova:cleanup { IMAGENAME } {
    exec /bin/rm -rf /tmp/$IMAGENAME.img
    exec /bin/rm -rf /tmp/$IMAGENAME.hdr
}
proc anova:editfile { w } {
    global PXHOME

    set filename [$w.non.file subwidget entry get]

    if { $filename=="" } {
	MxPause "Please enter a file name"
	return
    }
    if { [file exists $filename]==1 } {
	exec sh $PXHOME/bin/editconv.sh $filename $filename &
	MxPause "This window needs to be closed before any changes will take effect"
    } else {
	MxPause "File $filename does not exist"
	#exec touch $filename
	#exec sh $PXHOME/bin/editconv.sh $filename $filename &
    }
}

# ===== Main =====
global ANOVAvariables

# Need to save data locally
#MxGetCurrentFolder Folder
#MxSetFolderProperties $Folder {{SaveAll true}}

set ANOVAvariables(NUM) 0
set ANOVAvariables(thold) 10
set anova_disable_buttons 0

if {![info exists call_anova]} {
    anova:dialog .name
    tkwait window .name
}
