/*  ContrastMgr.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include <iostream>
#include <fstream>
#define WANT_STREAM
#define WANT_MATH

#include "newmatap.h"
#include "newmatio.h"
#include "Volume.h"

#if !defined(__ContrastMgr_h)
#define __ContrastMgr_h

#ifndef NO_NAMESPACE
using namespace NEWMAT;
using namespace TACO;
namespace TACO{
#endif

  class ContrastMgr
    {
    public:
      ContrastMgr(const Matrix& pcontrasts);

      void Load();
      void Save(const string& suffix);

      void ComputeVarCope();
      void ComputeCope();
      void ComputeNeff(bool removeEnds);
      void ComputeZStat();

      const ColumnVector& GetVarCope() const {return varcb;}
      const ColumnVector& GetCope() const {return cb;}
      const float GetDof() const {return dof;}
      
      void SetContrast(const int p_num, const int p_c_counter) 
	{
	  c = contrasts.Row(p_num).t();
	  c_counter = p_c_counter;
	}

    private:
      ContrastMgr();
      ContrastMgr(const ContrastMgr&);
      ContrastMgr& operator=(const ContrastMgr& p_ContrastMgr);

      void GetCorrection(Matrix& corr, const int ind);

      // Contrasts:
      const Matrix& contrasts;
      ColumnVector c;
      int c_counter;
      int numParams;

      // Loaded data:
      Matrix corrections;
      Matrix b;
      float dof;
      Volume sigmaSquareds;

      // Calculated date:
      Volume varcb;
      Volume cb;
      Volume neff;      
      Volume tstat;
      Volume zstat;

      // Other:
      int numTS;
    };


#ifndef NO_NAMESPACE
       }
#endif

#endif


