/*  FilmGlsOptions.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

#include <string>
#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>

#if !defined(__FilmGlsOptions_h)
#define __FilmGlsOptions_h

#ifndef NO_NAMESPACE
namespace TACO {
#endif

class FilmGlsOptions {
 public:
  static FilmGlsOptions& getInstance();
  ~FilmGlsOptions() { delete gopt; }

  string inputfname;
  string paradigmfname;
  string contrastfname;
  string epifname;
  string datadir;
  string susanpath;

  string neffsfname;
  string zscoresfname;
  
  bool verbose;
  bool rand;
  bool detrend;
  bool smoothACEst;
  bool fitAutoRegressiveModel;
  
  int trimscans;
  int thresh;
  int ms;
  int numrand;
  short maxshort;

  void parse_command_line(int argc, char** argv, ofstream& logfile);

 private:
  FilmGlsOptions();
  
  const FilmGlsOptions& operator=(FilmGlsOptions&);
  FilmGlsOptions(FilmGlsOptions&);
      
  static FilmGlsOptions* gopt;

  void print_usage(int argc, char *argv[]);
  
};

inline FilmGlsOptions& FilmGlsOptions::getInstance(){
  if(gopt == NULL)
    gopt = new FilmGlsOptions();
  
  return *gopt;
}

inline FilmGlsOptions::FilmGlsOptions()
{ 
  // set up defaults
  zscoresfname = "zstats";
  neffsfname = "neffs";
  datadir = "results";
  susanpath = "/usr/local/fsl/bin/susan_smooth";
  epifname = "epivolume";
  thresh = 0;
  contrastfname = "";
  inputfname = "";  
  paradigmfname = "";
  
  numrand = 0;
  ms = 4;
  maxshort = 32000;
  fitAutoRegressiveModel = false;
 
  smoothACEst = false;
  verbose = false;
  rand = false;
  detrend = false;
}

#ifndef NO_NAMESPACE
       }
#endif

#endif









