#

#   renderstats_proc.tcl - TCL proc for colour overlay rendering
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   WWW:      http://www.fmrib.ox.ac.uk/fsl
#   Email:    fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or (at
#   your option) any later version.
#   
#   This program is distributed in the hope that it will be useful, but in
#   order that the University as a charitable foundation protects its
#   assets for the benefit of its educational and research purposes, the
#   University makes clear that no condition is made or to be implied, nor
#   is any warranty given or to be implied, as to the accuracy of FSL, or
#   that it will be suitable for any particular purpose or for use under
#   any specific conditions.  Furthermore, the University disclaims all
#   responsibility for the use which is made of FSL.  See the GNU General
#   Public License for more details.
#   
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#   USA

proc render_proc { Folder colour_type output_type images imagelist } {
    global FSLDIR USER PROCID

    if { $imagelist == "" } {
	#{{{ setup display range for current image, if CALmin/max are set

MxGetCurrentImage OUT

MxGetHeaderInfo $OUT 0 OP

if { [ keylget OP AVWDsr.Value.IDCalMin.Value dmin ] && [ keylget OP AVWDsr.Value.IDCalMax.Value dmax ] } {
    puts "Display range $dmin $dmax"
    MxSetDisplayRange $OUT V $dmin $dmax
}

#}}}
    } else {
	#{{{ basic setups

set IsStats 0
set finalname "Rendered "
set OV "/tmp/${USER}_${PROCID}_overlay"

FSLSaveAs [ lindex $imagelist 0 ] AVW ${OV}_bg.hdr true
set thecommand "exec $FSLDIR/bin/overlay $colour_type $output_type ${OV}_bg [ lindex $imagelist 1 ] [ lindex $imagelist 2 ]"

#}}}
	#{{{ stats setup

for { set i 1 } { $i <= $images } { incr i 1 } {

    set si($i) [ lindex $imagelist [ expr $i * 3 ] ]
    FSLSaveAs $si($i) AVW ${OV}_s${i}.hdr true
    set thecommand "$thecommand ${OV}_s${i} [ lindex $imagelist [ expr $i * 3 + 1 ] ] [ lindex $imagelist [ expr $i * 3 + 2 ] ] "

    MxGetImageProperties $si($i) pl

    if { $i == 1 && $output_type == 0 } {
	set enc [ keylget pl Encoding ]
	set bpp [ keylget pl Bits ]
	MxGetDimensions $si($i) width height depth
	if { $enc == "R" && $bpp == 32 && $width < 150 } {
	    set IsStats 1
	}
    }

    if { $i > 1 } {
	append finalname " and "
    }

    append finalname "[ keylget pl Name ]"
}

#}}}
	#{{{ run the program

if { $IsStats } {
    MxDupPage $si(1) OUT
}

set thecommand "$thecommand ${OV}_out"

puts $thecommand
set result [ catch { eval $thecommand } ErrMsg ]
if {$result != 0} {
    puts "$ErrMsg"
}

MxOpenImage $Folder ${OV}_out.hdr tmp

MxGetHeaderInfo $tmp 0 OP

if { $IsStats } {
    MxExpr $OUT = $tmp
    MxDeletePage $tmp
} else {
    set OUT $tmp
}

if { [ keylget OP AVWDsr.Value.IDCalMin.Value dmin ] && [ keylget OP AVWDsr.Value.IDCalMax.Value dmax ] } {
    puts "Display range $dmin $dmax"
    MxSetDisplayRange $OUT V $dmin $dmax
}

keylset OProps Name "$finalname"
MxSetImageProperties $OUT $OProps

#}}}
    }
    #{{{ load LUTS

set typeflag ""
if { $colour_type } {
    set typeflag "t"
}

MxLoadMaterialMap $OUT ${FSLDIR}/etc/luts/render${images}${typeflag}.map
MxLoadLUT $OUT ${FSLDIR}/etc/luts/render${images}${typeflag}.lut

#}}}
}
